#!/bin/awk
#   Script : QAC.awk
#   Author : Richard A. O'Keefe
#   Updated: %G%
#   Purpose: Question-Answer Compiler : QA -> C89.

BEGIN {
    print "#include <ctype.h>"       # toupper()
    print "#include <stdio.h>"       # gets(), puts(), printf()
    print "#include <stdlib.h>"      # EXIT_SUCCESS, EXIT_FAILURE
    print "#include <string.h>"      # strchr()
    print ""
    print "static int right = 0;"
    print "static int wrong = 0;"
    print ""
    print "static void check(char const *expected) {"
    print "    char buffer[128];"
    print "    int i;"
    print "    int flag = 1;"
    print ""
 ## This used to generate a call to gets(), which is a security hole,
 ## but for this application, who cared?  MacOS X now whinges about
 ## it every time you run a program that uses gets(), whether it ever
 ## calls it or not, so now we generate a call to fgets(), which is a
 ## much nastier function to use.  In this program we get lucky.
    print "    if (fgets(buffer, sizeof buffer, stdin) == 0) abort();"
    print "    for (i = 0; buffer[i] != '\\0'; i++) {"
    print "        buffer[i] = toupper(buffer[i]);"
    print "    }"
    print "    for (i = 0; expected[i] != 0; i++) {"
    print "        if (strchr(buffer, expected[i]) == 0) flag = 0;"
    print "    }"
    print "    if (flag) {"
    print "        puts(" to_C_string("Correct.") ");"
    print "        right++;"
    print "    } else {"
    print "        printf(\"Sorry, the right answer is %s.\\n\", expected);"
    print "        wrong++;"
    print "    }"
    print "}"
    print ""
    print "int main(void) {"
}

function to_C_string(s) {
    gsub(/\\/, "\\\\", s)
    gsub(/"/,  "\\\"", s)
    gsub(/\t/, "\\t",  s)
    return "\"" s "\""
}

/^Q / {
    print "    puts(" to_C_string(substr($0, 3)) ");"
    position = 0
    expect = ""
    next
}

/^[DAFT] / {
    position++
    label = substr("ABCDEFGHIJKLMNOPQRSTUVWXYZ", position, 1)
    print "    puts(" to_C_string("(" label ") " substr($0, 3)) ");"
    if ($0 ~ /^[AT]/) expect = expect label
    next
}

/^E/ {
    print "    check(" to_C_string(expect) ");"
    print ""
    next
}

/^[^QDAFTER]/ {
    printf "Oh dear, there is an error at line %d of the script %s.\n" \
           "This is our fault, not yours.\n<%s>\n", FNR, FILENAME, $0  \
           >"/dev/stderr"
    exit 1
}

END {
    print "    printf(\"You got %d questions right out of %d.\\n\","
    print "        right, right+wrong);"
    print "    if (wrong == 0) puts(\"Congratulations!\");"
    print "    return wrong == 0 ? EXIT_SUCCESS : EXIT_FAILURE;"
    print "}"
    print ""
}
