#!/bin/awk
#   Script : QAF.awk
#   Author : Richard A. O'Keefe
#   Updated: February 2013
#   Purpose: Question-Answer Formatter
#   Usage  : mawk -f common.awk -f latex.awk -f QAF.awk QA.dat >QA.tex
#          : pdflatex QA.tex

#   Note:  this uses a small part (common.awk, latex.awk) of an existing
#   library for converting SGML/XML files to LaTeX/Troff/HTML.  This code
#   that was written for this example is just this file.

BEGIN {
    begin_document()   
    begin_numbered_list()
    state = "B"
}

/^Q / {
    if (state != "Q") {
	if (state == "E") end_item()
	begin_item()
	state = "Q"
    }
    put_text(substr($0, 3))
    position = 0
    expect = ""
    next
}

/^[DAFT] / {
    if (state == "Q") {
	begin_unordered_list()
	state = "A"
    } else {
	end_item()
    }
    position++
    label = substr("ABCDEFGHIJKLMNOPQRSTUVWXYZ", position, 1)
    begin_item("(" label ")")
    put_text(substr($0, 3))
    if ($0 ~ /^[AT]/) expect = expect label
    next
}

/^E/ {
    end_item()
    end_unordered_list()
    state = "E"
    next
}

/^[^QDAFTER]/ {
    printf "Oh dear, there is an error at line %d of the script %s.\n" \
           "This is our fault, not yours.\n<%s>\n", FNR, FILENAME, $0  \
           >"/dev/stderr"
    exit 1
}

END {
    end_item()
    end_numbered_list()
    end_document()
}
